@@ -17,6 +17,7 @@
 # License along with this program; if not, see <http://www.gnu.org/licenses/>.
 import os
 from gettext import gettext as _
+from typing import Optional
 
 import numpy
 from gi.repository import Gdk
@@ -35,12 +36,15 @@ from matplotlib.lines import Line2D
 
 from pitivi.configure import get_pixmap_dir
 from pitivi.effects import ALLOWED_ONLY_ONCE_EFFECTS
+from pitivi.timeline.markers import ClipMarkersBox
+from pitivi.timeline.markers import Marker
 from pitivi.timeline.previewers import AudioPreviewer
 from pitivi.timeline.previewers import ImagePreviewer
 from pitivi.timeline.previewers import TitlePreviewer
 from pitivi.timeline.previewers import VideoPreviewer
 from pitivi.undo.timeline import CommitTimelineFinalizingAction
 from pitivi.utils.loggable import Loggable
+from pitivi.utils.markers import MarkerListManager
 from pitivi.utils.misc import disconnect_all_by_func
 from pitivi.utils.misc import filename_from_uri
 from pitivi.utils.pipeline import PipelineError
@@ -49,9 +53,11 @@ from pitivi.utils.timeline import SELECT_ADD
 from pitivi.utils.timeline import Selected
 from pitivi.utils.timeline import UNSELECT
 from pitivi.utils.timeline import Zoomable
+from pitivi.utils.ui import CURSORS
+from pitivi.utils.ui import DRAG_CURSOR
 from pitivi.utils.ui import EFFECT_TARGET_ENTRY
-from pitivi.utils.ui import set_children_state_recurse
-from pitivi.utils.ui import unset_children_state_recurse
+from pitivi.utils.ui import NORMAL_CURSOR
+from pitivi.utils.ui import set_state_flags_recurse
 
 KEYFRAME_LINE_HEIGHT = 2
 KEYFRAME_LINE_ALPHA = 0.5
@@ -60,14 +66,6 @@ KEYFRAME_NODE_COLOR = "#F57900"  # "Tango" medium orange
 SELECTED_KEYFRAME_NODE_COLOR = "#204A87"  # "Tango" dark sky blue
 HOVERED_KEYFRAME_NODE_COLOR = "#3465A4"  # "Tango" medium sky blue
 
-CURSORS = {
-    GES.Edge.EDGE_START: Gdk.Cursor.new(Gdk.CursorType.LEFT_SIDE),
-    GES.Edge.EDGE_END: Gdk.Cursor.new(Gdk.CursorType.RIGHT_SIDE)
-}
-
-NORMAL_CURSOR = Gdk.Cursor.new(Gdk.CursorType.LEFT_PTR)
-DRAG_CURSOR = Gdk.Cursor.new(Gdk.CursorType.HAND1)
-
 
 def get_pspec(element_factory_name, propname):
     element = Gst.ElementFactory.make(element_factory_name)
@@ -99,13 +97,20 @@ class KeyframeCurve(FigureCanvasGTK3Cairo, Loggable):
         FigureCanvasGTK3Cairo.__init__(self, figure)
         Loggable.__init__(self)
 
+        # Remove the "matplotlib-canvas" class which forces a white background.
+        # https://github.com/matplotlib/matplotlib/commit/3c832377fb4c4b32fcbdbc60fdfedb57296bc8c0
+        style_ctx = self.get_style_context()
+        for css_class in style_ctx.list_classes():
+            style_ctx.remove_class(css_class)
+
+        style_ctx.add_class("KeyframeCurve")
+
         self._ges_elem = ges_elem
         self._timeline = timeline
         self.__source = binding.props.control_source
         self._connect_sources()
         self.__property_name = binding.props.name
         self.__paramspec = binding.pspec
-        self.get_style_context().add_class("KeyframeCurve")
 
         self.__ylim_min, self.__ylim_max = KeyframeCurve.YLIM_OVERRIDES.get(
             binding.pspec, (0.0, 1.0))
@@ -116,8 +121,8 @@ class KeyframeCurve(FigureCanvasGTK3Cairo, Loggable):
         self._line_xs = []
         self._line_ys = []
 
-        # facecolor to None for transparency
-        self._ax: Axes = figure.add_axes([0, 0, 1, 1], facecolor='None')
+        transparent = (0, 0, 0, 0)
+        self._ax: Axes = figure.add_axes([0, 0, 1, 1], facecolor=transparent)
         # Clear the Axes object.
         self._ax.cla()
         self._ax.grid(False)
@@ -321,6 +326,9 @@ class KeyframeCurve(FigureCanvasGTK3Cairo, Loggable):
                 self.handling_motion = True
             return
 
+        if event.guiEvent.type != Gdk.EventType.BUTTON_PRESS:
+            return
+
         result = self.__line.contains(event)
         if result[0]:
             # The line has been clicked.
@@ -651,19 +659,21 @@ class TimelineElement(Gtk.Layout, Zoomable, Loggable):
         self.__width = 0
         self.__height = 0
 
-        # Needed for effect's keyframe toggling
-        self._ges_elem.ui_element = self
-
         self.props.vexpand = True
 
-        self.__previewer = self._get_previewer()
-        if self.__previewer:
-            self.add(self.__previewer)
+        self.previewer = self._get_previewer()
+        if self.previewer:
+            self.add(self.previewer)
 
         self.__background = self._get_background()
         if self.__background:
             self.add(self.__background)
 
+        self.markers = ClipMarkersBox(self.app, self._ges_elem)
+        self._ges_elem.markers_manager.set_markers_box(self.markers)
+
+        self.add(self.markers)
+
         self.keyframe_curve = None
         self.__controlled_property = None
         self.show_all()
@@ -675,29 +685,26 @@ class TimelineElement(Gtk.Layout, Zoomable, Loggable):
 
     def update_previewer(self):
         """Refreshes the previewer widget."""
-        if self.__previewer:
-            self.__previewer.refresh()
+        if self.previewer:
+            self.previewer.refresh()
 
     def release(self):
-        if self.__previewer:
-            self.__previewer.release()
+        if self.previewer:
+            self.previewer.release()
+
+        if self.markers:
+            self._ges_elem.markers_manager.set_markers_box(None)
+            self.markers.release()
 
     # Public API
     def set_size(self, width, height):
         width = max(0, width)
         self.set_size_request(width, height)
 
-        if self.__previewer:
-            self.__previewer.set_size_request(width, height)
-
-        if self.__background:
-            self.__background.set_size_request(width, height)
-
-        if self.keyframe_curve:
-            self.keyframe_curve.set_size_request(width, height)
-
-        self.__width = width
-        self.__height = height
+        if self.__width != width or self.__height != height:
+            self.__width = width
+            self.__height = height
+            self.update_sizes_and_positions()
 
     def show_keyframes(self, ges_elem, prop):
         self.__set_keyframes(ges_elem, prop)
@@ -768,7 +775,7 @@ class TimelineElement(Gtk.Layout, Zoomable, Loggable):
         self.keyframe_curve.connect("leave", self.__curve_leave_cb)
         self.keyframe_curve.set_size_request(self.__width, self.__height)
         self.keyframe_curve.show()
-        self.__update_keyframe_curve_visibility()
+        self.__update_keyframe_curve()
 
     def __create_control_binding(self, element):
         """Creates the required ControlBinding and keyframes."""
@@ -795,31 +802,55 @@ class TimelineElement(Gtk.Layout, Zoomable, Loggable):
     def do_draw(self, cr):
         self.propagate_draw(self.__background, cr)
 
-        if self.__previewer:
-            self.propagate_draw(self.__previewer, cr)
+        if self.previewer:
+            self.propagate_draw(self.previewer, cr)
 
         if self.keyframe_curve and self.keyframe_curve.is_drawable():
             project = self.timeline.app.project_manager.current_project
             if project.pipeline.get_simple_state() != Gst.State.PLAYING:
                 self.propagate_draw(self.keyframe_curve, cr)
 
+        if self.markers and self.markers.is_drawable():
+            self.propagate_draw(self.markers, cr)
+
     # Callbacks
     def __selected_changed_cb(self, unused_selected, selected):
         if not self.keyframe_curve and self.__controlled_property and \
                 selected and len(self.timeline.selection) == 1:
             self.__create_keyframe_curve()
 
+        if self.previewer:
+            self.previewer.set_selected(selected)
+
+        self.update_sizes_and_positions()
+
+    def update_sizes_and_positions(self):
+        markers_height = self.markers.props.height_request
+        width = self.__width
+        height = self.__height
+
+        if self.__background:
+            self.__background.set_size_request(width, height)
+
+        if self.previewer:
+            self.previewer.set_size_request(width, height)
+
+        if self.markers:
+            self.markers.set_size_request(width, markers_height)
+
+        # Prevent keyframe curve from overlapping onto markers.
         if self.keyframe_curve:
-            self.__update_keyframe_curve_visibility()
+            self.keyframe_curve.set_size_request(self.__width, self.__height - markers_height)
+            self.__update_keyframe_curve()
 
-        if self.__previewer:
-            self.__previewer.set_selected(selected)
-
-    def __update_keyframe_curve_visibility(self):
+    def __update_keyframe_curve(self):
         """Updates the keyframes widget visibility by adding or removing it."""
         if self._ges_elem.selected and len(self.timeline.selection) == 1:
+            markers_height = self.markers.props.height_request
             if not self.keyframe_curve.get_parent():
-                self.add(self.keyframe_curve)
+                self.put(self.keyframe_curve, 0, markers_height)
+            else:
+                self.move(self.keyframe_curve, 0, markers_height)
         else:
             self.remove(self.keyframe_curve)
 
@@ -1139,7 +1170,7 @@ class Clip(Gtk.EventBox, Zoomable, Loggable):
 
     __gtype_name__ = "PitiviClip"
 
-    def __init__(self, layer, ges_clip):
+    def __init__(self, layer: GES.Layer, ges_clip: GES.Clip):
         Gtk.EventBox.__init__(self)
         Zoomable.__init__(self)
         Loggable.__init__(self)
@@ -1159,6 +1190,7 @@ class Clip(Gtk.EventBox, Zoomable, Loggable):
         self.ges_clip = ges_clip
         self.ges_clip.ui = self
         self.ges_clip.selected = Selected()
+        self.ges_clip.selected.selected = self.ges_clip in self.timeline.selection
 
         self.audio_widget = None
         self.video_widget = None
@@ -1170,6 +1202,8 @@ class Clip(Gtk.EventBox, Zoomable, Loggable):
             self._add_child(ges_timeline_element)
             self.__connect_to_child(ges_timeline_element)
 
+        set_state_flags_recurse(self, Gtk.StateFlags.SELECTED, are_set=self.ges_clip.selected)
+
         # Connect to Widget signals.
         self.connect("button-release-event", self._button_release_event_cb)
         self.connect("event", self._event_cb)
@@ -1274,6 +1308,15 @@ class Clip(Gtk.EventBox, Zoomable, Loggable):
                 parent_height != self._current_parent_height or \
                 layer != self._current_parent:
 
+            offset_px = self.ns_to_pixel(self.ges_clip.props.in_point)
+
+            for ges_timeline_element in self.ges_clip.get_children(False):
+                if not ges_timeline_element.ui:
+                    continue
+
+                if ges_timeline_element.ui.markers:
+                    ges_timeline_element.ui.markers.offset = offset_px
+
             layer.move(self, x, y)
             self.set_size_request(width, height)
 
@@ -1317,6 +1360,8 @@ class Clip(Gtk.EventBox, Zoomable, Loggable):
         self.update_position()
 
     def _button_release_event_cb(self, unused_widget, event):
+        self.debug("Button release event")
+
         if self.timeline.got_dragged:
             # This means a drag & drop operation just finished and
             # this button-release-event should be ignored.
@@ -1330,7 +1375,7 @@ class Clip(Gtk.EventBox, Zoomable, Loggable):
 
         mode = SELECT
         if self.timeline.get_parent().control_mask:
-            if not self.get_state_flags() & Gtk.StateFlags.SELECTED:
+            if not self.ges_clip.selected:
                 mode = SELECT_ADD
             else:
                 mode = UNSELECT
@@ -1368,18 +1413,22 @@ class Clip(Gtk.EventBox, Zoomable, Loggable):
             handle.hide()
 
     def _event_cb(self, element, event):
+        prelight = None
         if (event.type == Gdk.EventType.ENTER_NOTIFY and
                 event.mode == Gdk.CrossingMode.NORMAL and
                 not self.timeline.scrubbing):
-            set_children_state_recurse(self, Gtk.StateFlags.PRELIGHT)
+            prelight = True
             for handle in self.handles:
                 handle.enlarge()
         elif (event.type == Gdk.EventType.LEAVE_NOTIFY and
               event.mode == Gdk.CrossingMode.NORMAL):
-            unset_children_state_recurse(self, Gtk.StateFlags.PRELIGHT)
+            prelight = False
             for handle in self.handles:
                 handle.shrink()
 
+        if prelight is not None:
+            set_state_flags_recurse(self, Gtk.StateFlags.PRELIGHT, are_set=prelight, ignored_classes=(Marker,))
+
         return False
 
     def _start_changed_cb(self, clip, pspec):
@@ -1406,11 +1455,12 @@ class Clip(Gtk.EventBox, Zoomable, Loggable):
     def __curve_leave_cb(self, unused_keyframe_curve):
         self.__show_handles()
 
-    def _add_child(self, ges_timeline_element):
+    def _add_child(self, ges_timeline_element: GES.TimelineElement):
         ges_timeline_element.selected = Selected()
+        ges_timeline_element.selected.selected = self.ges_clip.selected.selected
         ges_timeline_element.ui = None
 
-    def _child_added_cb(self, unused_ges_clip, ges_timeline_element):
+    def _child_added_cb(self, ges_clip, ges_timeline_element: GES.TimelineElement):
         self.__force_position_update = True
         self._add_child(ges_timeline_element)
         self.__connect_to_child(ges_timeline_element)
@@ -1419,7 +1469,7 @@ class Clip(Gtk.EventBox, Zoomable, Loggable):
     def _remove_child(self, ges_timeline_element):
         pass
 
-    def _child_removed_cb(self, unused_ges_clip, ges_timeline_element):
+    def _child_removed_cb(self, unused_ges_clip, ges_timeline_element: GES.TimelineElement):
         self.__force_position_update = True
         self.__disconnect_from_child(ges_timeline_element)
         self._remove_child(ges_timeline_element)
@@ -1429,7 +1479,7 @@ class Clip(Gtk.EventBox, Zoomable, Loggable):
 class SourceClip(Clip):
     __gtype_name__ = "PitiviSourceClip"
 
-    def __init__(self, layer, ges_clip):
+    def __init__(self, layer: GES.Layer, ges_clip: GES.Clip):
         Clip.__init__(self, layer, ges_clip)
 
     def _setup_widget(self):
@@ -1437,6 +1487,33 @@ class SourceClip(Clip):
 
         self.get_style_context().add_class("Clip")
 
+    def _add_child(self, ges_timeline_element: GES.TimelineElement):
+        Clip._add_child(self, ges_timeline_element)
+
+        # In some cases a GESEffect is added here,
+        # so we have to limit the markers initialization to GESSources.
+        if not isinstance(ges_timeline_element, GES.Source):
+            return
+
+        ges_source: GES.Source = ges_timeline_element
+
+        if not hasattr(ges_source, "markers_manager"):
+            ges_source.markers_manager = MarkerListManager(self.app.settings, ges_source)
+
+        widget = self._create_child_widget(ges_source)
+        if not widget:
+            return
+
+        ges_source.ui = widget
+        if ges_source.get_track_type() == GES.TrackType.VIDEO:
+            self._elements_container.pack_start(widget, expand=True, fill=False, padding=0)
+        else:
+            self._elements_container.pack_end(widget, expand=True, fill=False, padding=0)
+        widget.set_visible(True)
+
+    def _create_child_widget(self, ges_source: GES.Source) -> Optional[Gtk.Widget]:
+        raise NotImplementedError()
+
     def _remove_child(self, ges_timeline_element):
         if ges_timeline_element.ui:
             self._elements_container.remove(ges_timeline_element.ui)
@@ -1446,7 +1523,7 @@ class SourceClip(Clip):
 class UriClip(SourceClip):
     __gtype_name__ = "PitiviUriClip"
 
-    def __init__(self, layer, ges_clip):
+    def __init__(self, layer: GES.Layer, ges_clip: GES.Clip):
         SourceClip.__init__(self, layer, ges_clip)
         self.get_style_context().add_class("UriClip")
         self.props.has_tooltip = True
@@ -1457,69 +1534,52 @@ class UriClip(SourceClip):
 
         return True
 
-    def _add_child(self, ges_timeline_element):
-        SourceClip._add_child(self, ges_timeline_element)
+    def _create_child_widget(self, ges_source: GES.Source) -> Optional[Gtk.Widget]:
+        if ges_source.get_track_type() == GES.TrackType.AUDIO:
+            self.audio_widget = AudioUriSource(ges_source, self.timeline)
+            return self.audio_widget
+        elif ges_source.get_track_type() == GES.TrackType.VIDEO:
+            self.video_widget = VideoUriSource(ges_source, self.timeline)
+            return self.video_widget
 
-        if not isinstance(ges_timeline_element, GES.Source):
-            return
-
-        if ges_timeline_element.get_track_type() == GES.TrackType.AUDIO:
-            self.audio_widget = AudioUriSource(ges_timeline_element, self.timeline)
-            ges_timeline_element.ui = self.audio_widget
-            self._elements_container.pack_end(self.audio_widget, True, False, 0)
-            self.audio_widget.set_visible(True)
-        elif ges_timeline_element.get_track_type() == GES.TrackType.VIDEO:
-            self.video_widget = VideoUriSource(ges_timeline_element, self.timeline)
-            ges_timeline_element.ui = self.video_widget
-            self._elements_container.pack_start(self.video_widget, True, False, 0)
-            self.video_widget.set_visible(True)
+        return None
 
 
 class TestClip(SourceClip):
     __gtype_name__ = "PitiviTestClip"
 
-    def __init__(self, layer, ges_clip):
+    def __init__(self, layer: GES.Layer, ges_clip: GES.Clip):
         SourceClip.__init__(self, layer, ges_clip)
         self.get_style_context().add_class("TestClip")
 
-    def _add_child(self, ges_timeline_element):
-        SourceClip._add_child(self, ges_timeline_element)
+    def _create_child_widget(self, ges_source: GES.Source) -> Optional[Gtk.Widget]:
+        if ges_source.get_track_type() == GES.TrackType.VIDEO:
+            self.video_widget = VideoTestSource(ges_source, self.timeline)
+            return self.video_widget
 
-        if not isinstance(ges_timeline_element, GES.Source):
-            return
-
-        if ges_timeline_element.get_track_type() == GES.TrackType.VIDEO:
-            self.video_widget = VideoTestSource(ges_timeline_element, self.timeline)
-            ges_timeline_element.ui = self.video_widget
-            self._elements_container.pack_start(self.video_widget, True, False, 0)
-            self.video_widget.set_visible(True)
+        return None
 
 
 class TitleClip(SourceClip):
     __gtype_name__ = "PitiviTitleClip"
 
-    def __init__(self, layer, ges_clip):
+    def __init__(self, layer: GES.Layer, ges_clip: GES.Clip):
         SourceClip.__init__(self, layer, ges_clip)
         self.get_style_context().add_class("TitleClip")
 
-    def _add_child(self, ges_timeline_element):
-        SourceClip._add_child(self, ges_timeline_element)
+    def _create_child_widget(self, ges_source: GES.Source) -> Optional[Gtk.Widget]:
+        if ges_source.get_track_type() == GES.TrackType.VIDEO:
+            self.video_widget = TitleSource(ges_source, self.timeline)
+            return self.video_widget
 
-        if not isinstance(ges_timeline_element, GES.Source):
-            return
-
-        if ges_timeline_element.get_track_type() == GES.TrackType.VIDEO:
-            self.video_widget = TitleSource(ges_timeline_element, self.timeline)
-            ges_timeline_element.ui = self.video_widget
-            self._elements_container.pack_start(self.video_widget, True, False, 0)
-            self.video_widget.set_visible(True)
+        return None
 
 
 class TransitionClip(Clip):
 
     __gtype_name__ = "PitiviTransitionClip"
 
-    def __init__(self, layer, ges_clip):
+    def __init__(self, layer: GES.Layer, ges_clip: GES.Clip):
         self.__has_video = False
 
         Clip.__init__(self, layer, ges_clip)
