@@ -15,6 +15,9 @@
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this program; if not, see <http://www.gnu.org/licenses/>.
+from typing import Iterable
+from typing import Set
+
 from gi.repository import GES
 from gi.repository import GObject
 from gi.repository import Gst
@@ -70,11 +73,9 @@ class Selected(GObject.Object):
 class Selection(GObject.Object, Loggable):
     """Manages a set of clips representing a selection.
 
-    Attributes:
-        selected (List[GES.TrackElement]): Set of selected elements.
-
     Signals:
-        selection-changed: The contents of the selection changed.
+        selection-changed: The contents of the selection or the grouping of
+            the selected clips changed.
     """
 
     __gsignals__ = {
@@ -84,79 +85,77 @@ class Selection(GObject.Object, Loggable):
     def __init__(self):
         GObject.Object.__init__(self)
         Loggable.__init__(self)
-        self.selected = set()
+        self._clips: Set[GES.Clip] = set()
         self.can_group = False
         self.can_ungroup = False
 
-    def set_selection(self, objs, mode):
+    def set_selection(self, clips: Iterable[GES.Clip], mode: int):
         """Updates the current selection.
 
         Args:
-            objs (List[GES.TrackElement]): Timeline objects to update the
-                selection with.
+            clips (Iterable[GES.Clip]): Timeline clips to update the selection with.
             mode (SELECT or UNSELECT or SELECT_ADD): The type of update to
                 apply. The selection will be:
                 - `SELECT` : set to the provided selection.
                 - `UNSELECT` : the same minus the provided selection.
                 - `SELECT_ADD` : extended with the provided selection.
         """
-        selection = set()
-        for obj in objs:
-            # FIXME GES break, handle the fact that we have unlinked objects in
-            # GES
-            if isinstance(obj, GES.TrackElement):
-                selection.add(obj.get_parent())
-            else:
-                selection.add(obj)
+        if not isinstance(clips, Set):
+            clips = set(clips)
+        self.debug("Updating selection %s mode %s", clips, mode)
         if mode == SELECT_ADD:
-            selection = self.selected | selection
+            selection = self._clips | clips
         elif mode == UNSELECT:
-            selection = self.selected - selection
+            selection = self._clips - clips
+        else:
+            selection = clips
 
-        old_selection = self.selected
-        if selection == old_selection:
-            # Nothing changed. This can happen for example when the user clicks
-            # the selected clip, then the clip remains selected.
-            return
-        self.selected = selection
+        selection_changed = self._clips != selection
+        if selection_changed:
+            old_selection = self._clips
+            self._clips = selection
 
-        for obj, selected in self.__get_selection_changes(old_selection):
-            obj.selected.selected = selected
-            if obj.ui:
-                if selected:
-                    from pitivi.utils.ui import set_children_state_recurse
-                    set_children_state_recurse(obj.ui, Gtk.StateFlags.SELECTED)
-                else:
-                    from pitivi.utils.ui import unset_children_state_recurse
-                    unset_children_state_recurse(obj.ui, Gtk.StateFlags.SELECTED)
-            for element in obj.get_children(False):
-                if isinstance(obj, (GES.BaseEffect, GES.TextOverlay)):
-                    continue
-                element.selected.selected = selected
-        self.set_can_group_ungroup()
-        self.emit("selection-changed")
+            for obj, selected in self.__get_selection_changes(old_selection):
+                obj.selected.selected = selected
+                if obj.ui:
+                    from pitivi.utils.ui import set_state_flags_recurse
+                    set_state_flags_recurse(obj.ui, Gtk.StateFlags.SELECTED, are_set=selected)
 
-    def set_can_group_ungroup(self):
-        can_ungroup = False
-        toplevels = self.toplevels
-        self.can_group = len(toplevels) > 1
+                for element in obj.get_children(False):
+                    if isinstance(obj, (GES.BaseEffect, GES.TextOverlay)):
+                        continue
+                    element.selected.selected = selected
+
+        # Always check, maybe the clips have been grouped or ungrouped.
+        grouping_changed = self._set_can_group_ungroup()
+
+        if selection_changed or grouping_changed:
+            self.emit("selection-changed")
+
+    def _set_can_group_ungroup(self) -> bool:
+        toplevels = self.toplevels()
+        can_group = len(toplevels) > 1
         # If we allow grouping, we disallow ungrouping.
-        if not self.can_group:
+        can_ungroup = False
+        if not can_group:
             for toplevel in toplevels:
                 if (isinstance(toplevel, GES.Group) or
                         (isinstance(toplevel, GES.Container) and len(toplevel.get_children(False)) > 1)):
                     can_ungroup = True
                     break
-        self.can_ungroup = can_ungroup and not self.can_group
+        changed = (self.can_group, self.can_ungroup) != (can_group, can_ungroup)
+        self.can_group = can_group
+        self.can_ungroup = can_ungroup
+        return changed
 
     def __get_selection_changes(self, old_selection):
-        for obj in old_selection - self.selected:
+        for obj in old_selection - self._clips:
             yield obj, False
 
         # Announce all selected objects that they are selected, even if
         # they were already selected. This allows them to update based on
         # the current selection.
-        for obj in self.selected:
+        for obj in self._clips:
             yield obj, True
 
     def select(self, objs):
@@ -172,7 +171,7 @@ class Selection(GObject.Object, Loggable):
             List[GES.TrackElement]
         """
         objects = []
-        for clip in self.selected:
+        for clip in self._clips:
             objects.extend(clip.get_children(False))
 
         return set(objects)
@@ -183,17 +182,16 @@ class Selection(GObject.Object, Loggable):
         Args:
             clip_type (type): The class the clip must be an instance of.
         """
-        if len(self.selected) == 1:
-            clip = tuple(self.selected)[0]
+        if len(self._clips) == 1:
+            clip = tuple(self._clips)[0]
             if isinstance(clip, clip_type):
                 return clip
         return None
 
-    @property
     def toplevels(self):
         """Returns the toplevel elements of the selection."""
         toplevels = set()
-        for obj in self.selected:
+        for obj in self._clips:
             if not obj.timeline:
                 # The element has been removed from the timeline. Ignore it.
                 continue
@@ -219,11 +217,16 @@ class Selection(GObject.Object, Loggable):
             have only non-serializable ancestors.
         """
         toplevels = set()
-        for obj in self.selected:
-            if not obj.timeline:
-                # The element has been removed from the timeline. Ignore it.
+        for ges_clip in self._clips:
+            if not ges_clip.timeline:
+                # The clip has been removed from the timeline. Ignore it.
                 continue
-            toplevel = obj.get_toplevel_parent()
+
+            if isinstance(ges_clip, GES.Transition):
+                # The transitions are updated automatically, no need to group them etc.
+                continue
+
+            toplevel = ges_clip.get_toplevel_parent()
             toplevels.add(toplevel)
 
         group = GES.Group.new()
@@ -233,10 +236,20 @@ class Selection(GObject.Object, Loggable):
         return group
 
     def __len__(self):
-        return len(self.selected)
+        return len(self._clips)
 
     def __iter__(self):
-        return iter(self.selected)
+        return iter(self._clips)
+
+    @staticmethod
+    def get_clips_of(containers: Iterable[GES.Container]) -> Iterable[GES.Clip]:
+        for container in containers:
+            if isinstance(container, GES.Clip):
+                yield container
+            elif isinstance(container, GES.Container):
+                for child in container.get_children(recursive=True):
+                    if isinstance(child, GES.Clip):
+                        yield child
 
 
 class EditingContext(GObject.Object, Loggable):
@@ -257,7 +270,7 @@ class EditingContext(GObject.Object, Loggable):
         app (Pitivi): The app.
     """
 
-    def __init__(self, focus, timeline, mode, edge, app, log_actions):
+    def __init__(self, focus, timeline, mode, edge, app, log_actions=None):
         GObject.Object.__init__(self)
         Loggable.__init__(self)
         if isinstance(focus, GES.TrackElement):
@@ -273,7 +286,6 @@ class EditingContext(GObject.Object, Loggable):
         self.old_priority = self.focus.get_priority()
 
         self.new_position = None
-        self.new_priority = None
 
         self.timeline = timeline
         self.app = app
@@ -290,6 +302,7 @@ class EditingContext(GObject.Object, Loggable):
                                       toplevel=True)
 
     def finish(self):
+        self.debug("Finishing editing context")
         if self.__log_actions:
             self.app.action_log.commit("move-clip")
         self.timeline.get_asset().pipeline.commit_timeline()
@@ -318,7 +331,6 @@ class EditingContext(GObject.Object, Loggable):
             priority = max(0, priority)
 
         self.new_position = position
-        self.new_priority = priority
 
         if self.with_video:
             frame = self.timeline.get_frame_at(position)
@@ -456,6 +468,7 @@ class Zoomable:
         # DIE YOU CUNTMUNCH CLOCK_TIME_NONE UBER STUPIDITY OF CRACK BINDINGS !!
         if duration == Gst.CLOCK_TIME_NONE:
             return 0
+
         return int((float(duration) / Gst.SECOND) * cls.zoomratio)
 
     @classmethod
@@ -465,6 +478,7 @@ class Zoomable:
         # DIE YOU CUNTMUNCH CLOCK_TIME_NONE UBER STUPIDITY OF CRACK BINDINGS !!
         if duration == Gst.CLOCK_TIME_NONE:
             return 0
+
         return (float(duration) / Gst.SECOND) * cls.zoomratio
 
     def zoom_changed(self):
