@@ -13,9 +13,8 @@
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
-# License along with this program; if not, write to the
-# Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
-# Boston, MA 02110-1301, USA.
+# License along with this program; if not, see <http://www.gnu.org/licenses/>.
+# pylint: disable=protected-access
 import locale
 import subprocess
 from unittest import mock
@@ -39,19 +38,14 @@ try:
 except ImportError:
     GstValidate = None
 except ValueError:
+    # pylint: disable=invalid-name
     GstValidate = None
 
 monitor = None
 has_validate = False
 
 
-# FIXME Remove when we have a hard dep on Gst 1.14.
-def get_pipeline(scenario):
-    return getattr(scenario, "pipeline",
-                   scenario.get_pipeline())
-
-
-def Event(event_type, **kwargs):
+def create_event(event_type, **kwargs):
     event_types_constructors = {
         Gdk.EventType.BUTTON_PRESS: Gdk.EventButton,
         Gdk.EventType.BUTTON_RELEASE: Gdk.EventButton,
@@ -68,26 +62,24 @@ def Event(event_type, **kwargs):
 
     return event
 
+
 if GstValidate:
     class PitiviMonitor(GstValidate.Monitor):
-        def __init__(self, runner, object):
-            GstValidate.Monitor.__init__(self, object=object, validate_runner=runner)
+        def __init__(self, runner, obj):
+            GstValidate.Monitor.__init__(self, object=obj, validate_runner=runner)
 
             if GstValidate:
                 try:
-                    import gi
                     gi.require_version('Wnck', '3.0')
                     from gi.repository import Wnck
-                    Wnck.Screen.get_default().connect("window-opened", self._windowOpenedCb)
+                    Wnck.Screen.get_default().connect("window-opened", self._window_opened_cb)
                 except (ImportError, ValueError):
                     print("Wnck not present on the system,"
                           " not checking the sink does not open a new window")
-                    pass
                 except AttributeError:
                     print("Wnck can not be used on the system")
-                    pass
 
-        def _windowOpenedCb(self, screen, window):
+        def _window_opened_cb(self, screen, window):
             global monitor
 
             if window.get_name() == 'renderer' and monitor:
@@ -95,7 +87,7 @@ if GstValidate:
                                       "New window created by the sink,"
                                       " that should not happen")
 
-        def checkWrongWindow(self):
+        def check_wrong_window(self):
             try:
                 windows = subprocess.check_output(["xwininfo", "-tree", "-root"]).decode(locale.getdefaultlocale()[1])
                 for w in windows.split('\n'):
@@ -118,21 +110,21 @@ def create_monitor(runner, app):
         GstValidate.Reporter.set_name(monitor, "Pitivi")
 
 
-def stop(scenario, action):
+def stop_func(scenario, action):
     global monitor
 
     if monitor:
-        monitor.checkWrongWindow()
+        monitor.check_wrong_window()
 
     if action.structure.get_boolean("force")[0]:
         GstValidate.execute_action(GstValidate.get_action_type(action.type).overriden_type,
                                    action)
 
-        timeline = get_pipeline(scenario).props.timeline
+        timeline = scenario.get_pipeline().props.timeline
         project = timeline.get_asset()
 
         if project:
-            project.setModificationState(False)
+            project.set_modification_state(False)
             GstValidate.print_action(action, "Force quitting, ignoring any"
 
                                      " changes in the project\n")
@@ -145,8 +137,8 @@ def stop(scenario, action):
     return 1
 
 
-def positionChangedCb(pipeline, position, scenario, action,
-                      wanted_position):
+def position_changed_cb(pipeline, position, scenario, action,
+                        wanted_position):
     if pipeline._busy_async:
         return
 
@@ -155,27 +147,27 @@ def positionChangedCb(pipeline, position, scenario, action,
 
     print(str(wanted_position), str(position))
     if wanted_position != position:
-        scenario.report_simple(GLib.quark_from_string(
-            "scenario::execution-error"),
-            "Position after seek (%s) does not match wanted "
-            "one %s" % (Gst.TIME_ARGS(position),
-                        Gst.TIME_ARGS(wanted_position)))
+        scenario.report_simple(
+            GLib.quark_from_string("scenario::execution-error"),
+            "Position after seek (%s) does not match wanted one %s" % (
+                Gst.TIME_ARGS(position), Gst.TIME_ARGS(wanted_position)))
 
-    pipeline.disconnect_by_func(positionChangedCb)
+    pipeline.disconnect_by_func(position_changed_cb)
     action.set_done()
 
 
-def seek(scenario, action):
+def seek_func(scenario, action):
     res, wanted_position = GstValidate.utils_get_clocktime(action.structure,
-                                                      "start")
-    get_pipeline(scenario).simple_seek(wanted_position)
-    get_pipeline(scenario).connect("position", positionChangedCb, scenario,
-                              action, wanted_position)
+                                                           "start")
+    assert res
+    scenario.get_pipeline().simple_seek(wanted_position)
+    scenario.get_pipeline().connect("position", position_changed_cb, scenario,
+                                    action, wanted_position)
 
     return GstValidate.ActionReturn.ASYNC
 
 
-def set_state(scenario, action):
+def set_state_func(scenario, action):
     wanted_state = action.structure["state"]
     if wanted_state is None:
         wanted_state = action.structure.get_name()
@@ -186,7 +178,6 @@ def set_state(scenario, action):
 
     if wanted_state == "paused":
         if scenario.__dict__.get("started", None) is None:
-
             return 1
 
     return GstValidate.execute_action(GstValidate.get_action_type(action.type).overriden_type,
@@ -207,14 +198,16 @@ def get_edge(structure):
     return edge
 
 
-def _releaseButtonIfNeeded(scenario, action, timeline, container, edge, layer_prio,
-                           position, y):
-    try:
-        next_actions = scenario.get_actions()
-        for next_action in next_actions[1:]:
-            if next_action.type not in ["wait", "add-layer"]:
-                break
-    except KeyError:
+def _release_button_if_needed(scenario, timeline, container, layer_prio,
+                              position, y):
+    next_actions = scenario.get_actions()
+    if len(next_actions) == 1:
+        return
+    next_action = None
+    for next_action in next_actions[1:]:
+        if next_action.type not in ["wait", "add-layer"]:
+            break
+    if not next_action:
         return
 
     need_release = True
@@ -226,8 +219,8 @@ def _releaseButtonIfNeeded(scenario, action, timeline, container, edge, layer_pr
 
     if next_action is None or need_release:
         scenario.dragging = False
-        x = Zoomable.nsToPixelAccurate(position)
-        event = Event(Gdk.EventType.BUTTON_RELEASE, button=1, x=x, y=y)
+        x = Zoomable.ns_to_pixel_accurate(position)
+        event = create_event(Gdk.EventType.BUTTON_RELEASE, button=1, x=x, y=y)
         with mock.patch.object(Gtk, "get_event_widget") as get_event_widget:
             get_event_widget.return_value = container.ui
             container.ui._button_release_event_cb(None, event)
@@ -239,21 +232,21 @@ def _releaseButtonIfNeeded(scenario, action, timeline, container, edge, layer_pr
                                    % (container.get_layer().get_priority(),
                                       layer_prio))
 
-        cleanEditModes(timeline, scenario)
+        clean_edit_modes(timeline, scenario)
 
 
-def cleanEditModes(timeline, scenario):
+def clean_edit_modes(timeline, scenario):
     if scenario.last_mode == GES.EditMode.EDIT_RIPPLE:
-        event = Event(Gdk.EventType.KEY_RELEASE, keyval=Gdk.KEY_Shift_L)
+        event = create_event(Gdk.EventType.KEY_RELEASE, keyval=Gdk.KEY_Shift_L)
         timeline.ui.get_parent().do_key_release_event(event)
     elif scenario.last_mode == GES.EditMode.EDIT_ROLL:
-        event = Event(Gdk.EventType.KEY_RELEASE, keyval=Gdk.KEY_Control_L)
+        event = create_event(Gdk.EventType.KEY_RELEASE, keyval=Gdk.KEY_Control_L)
         timeline.ui.get_parent().do_key_release_event(event)
 
     scenario.last_mode = None
 
 
-def setEditingMode(timeline, scenario, action):
+def set_editing_mode(timeline, scenario, action):
     if not hasattr(scenario, "last_mode"):
         scenario.last_mode = None
 
@@ -267,24 +260,24 @@ def setEditingMode(timeline, scenario, action):
         mode = GES.EditMode.EDIT_NORMAL
 
     if mode == GES.EditMode.EDIT_RIPPLE:
-        timeline.ui.get_parent().do_key_press_event(Event(Gdk.EventType.KEY_PRESS, keyval=Gdk.KEY_Shift_L))
+        timeline.ui.get_parent().do_key_press_event(create_event(Gdk.EventType.KEY_PRESS, keyval=Gdk.KEY_Shift_L))
 
         if scenario.last_mode == GES.EditMode.EDIT_ROLL:
-            timeline.ui.get_parent().do_key_release_event(Event(Gdk.EventType.KEY_RELEASE, keyval=Gdk.KEY_Control_L))
+            timeline.ui.get_parent().do_key_release_event(create_event(Gdk.EventType.KEY_RELEASE, keyval=Gdk.KEY_Control_L))
 
     elif mode == GES.EditMode.EDIT_ROLL:
-        timeline.ui.do_key_press_event(Event(Gdk.EventType.KEY_PRESS, keyval=Gdk.KEY_Control_L))
+        timeline.ui.do_key_press_event(create_event(Gdk.EventType.KEY_PRESS, keyval=Gdk.KEY_Control_L))
 
         if scenario.last_mode == GES.EditMode.EDIT_RIPPLE:
-            timeline.ui.do_key_release_event(Event(Gdk.EventType.KEY_RELEASE, keyval=Gdk.KEY_Shift_L))
+            timeline.ui.do_key_release_event(create_event(Gdk.EventType.KEY_RELEASE, keyval=Gdk.KEY_Shift_L))
     else:
-        cleanEditModes(timeline, scenario)
+        clean_edit_modes(timeline, scenario)
 
     scenario.last_mode = mode
 
 
-def editContainer(scenario, action):
-    timeline = get_pipeline(scenario).props.timeline
+def edit_container_func(scenario, action):
+    timeline = scenario.get_pipeline().props.timeline
     container = timeline.get_element(action.structure["container-name"])
 
     if container is None:
@@ -307,7 +300,7 @@ def editContainer(scenario, action):
     edge = get_edge(action.structure)
     container_ui = container.ui
 
-    setEditingMode(timeline, scenario, action)
+    set_editing_mode(timeline, scenario, action)
 
     y = 21 - container_ui.translate_coordinates(timeline.ui, 0, 0)[1]
 
@@ -340,70 +333,65 @@ def editContainer(scenario, action):
         event_widget = container.ui
         if isinstance(container, GES.SourceClip):
             if edge == GES.Edge.EDGE_START:
-                event_widget = container.ui.leftHandle
+                event_widget = container.ui.left_handle
             elif edge == GES.Edge.EDGE_END:
-                event_widget = container.ui.rightHandle
+                event_widget = container.ui.right_handle
 
         scenario.dragging = True
-        event = Event(Gdk.EventType.BUTTON_PRESS, button=1, y=y)
+        event = create_event(Gdk.EventType.BUTTON_PRESS, button=1, y=y)
         with mock.patch.object(Gtk, "get_event_widget") as get_event_widget:
             get_event_widget.return_value = event_widget
             timeline.ui._button_press_event_cb(event_widget, event)
 
-    event = Event(Gdk.EventType.MOTION_NOTIFY, button=1,
-                  x=Zoomable.nsToPixelAccurate(position) -
-                  container_ui.translate_coordinates(timeline.ui.layout.layers_vbox, 0, 0)[0],
-                  y=y, state=Gdk.ModifierType.BUTTON1_MASK)
+    x = Zoomable.ns_to_pixel_accurate(position) - container_ui.translate_coordinates(timeline.ui.layout.layers_vbox, 0, 0)[0]
+    event = create_event(Gdk.EventType.MOTION_NOTIFY, button=1,
+                         x=x, y=y, state=Gdk.ModifierType.BUTTON1_MASK)
     with mock.patch.object(Gtk, "get_event_widget") as get_event_widget:
         get_event_widget.return_value = container.ui
         timeline.ui._motion_notify_event_cb(None, event)
 
-    GstValidate.print_action(action, "Editing %s to %s in %s mode, edge: %s "
+    GstValidate.print_action(action,
+                             "Editing %s to %s in %s mode, edge: %s "
                              "with new layer prio: %d\n" % (action.structure["container-name"],
                                                             Gst.TIME_ARGS(position),
                                                             scenario.last_mode,
                                                             edge,
                                                             layer_prio))
 
-    _releaseButtonIfNeeded(scenario, action, timeline, container, edge, layer_prio,
-                           position, y)
+    _release_button_if_needed(scenario, timeline, container, layer_prio,
+                              position, y)
     scenario.last_edge = edge
 
     return 1
 
 
-# def commit(scenario, action):
-
-#     return True
-
-
-def split_clip(scenario, action):
-    timeline = get_pipeline(scenario).props.timeline.ui
-    timeline.get_parent()._splitCb(None, None)
+def split_clip_func(scenario, action):
+    timeline = scenario.get_pipeline().props.timeline.ui
+    timeline.get_parent()._split_cb(None, None)
 
     return True
 
 
-def zoom(scenario, action):
-    timeline = get_pipeline(scenario).props.timeline
+def zoom_func(scenario, action):
+    timeline = scenario.get_pipeline().props.timeline
 
     GstValidate.print_action(action, action.type.replace('-', ' ') + "\n")
 
     {"zoom-fit": timeline.ui.set_best_zoom_ratio,
-     "zoom-out": Zoomable.zoomOut,
-     "zoom-in": Zoomable.zoomIn}[action.type]()
+     "zoom-out": Zoomable.zoom_out,
+     "zoom-in": Zoomable.zoom_in}[action.type]()
 
     return True
 
 
-def setZoomLevel(scenario, action):
-    Zoomable.setZoomLevel(action.structure["level"])
+def set_zoom_level_func(scenario, action):
+    Zoomable.set_zoom_level(action.structure["level"])
 
     return True
 
 
-def add_layer(scenario, action):
-    timeline = get_pipeline(scenario).props.timeline
+def add_layer_func(scenario, action):
+    timeline = scenario.get_pipeline().props.timeline
     if len(timeline.get_layers()) == 0:
         GstValidate.print_action(action, "Adding first layer\n")
         timeline.append_layer()
@@ -413,7 +401,7 @@ def add_layer(scenario, action):
     return True
 
 
-def remove_clip(scenario, action):
+def remove_clip_func(scenario, action):
     try:
         next_action = scenario.get_actions()[1]
     except KeyError:
@@ -432,9 +420,9 @@ def remove_clip(scenario, action):
     return GstValidate.execute_action(action_type.overriden_type, action)
 
 
-def select_clips(scenario, action):
+def select_clips_func(scenario, action):
     should_select = True
-    timeline = get_pipeline(scenario).props.timeline
+    timeline = scenario.get_pipeline().props.timeline
     clip = timeline.get_element(action.structure["clip-name"])
 
     if clip is None:
@@ -452,10 +440,10 @@ def select_clips(scenario, action):
         if clip.ui.get_state_flags() & Gtk.StateFlags.SELECTED:
             should_select = False
 
-        event = Event(Gdk.EventType.KEY_PRESS, keyval=Gdk.KEY_Control_L)
+        event = create_event(Gdk.EventType.KEY_PRESS, keyval=Gdk.KEY_Control_L)
         timeline.ui.get_parent().do_key_press_event(event)
 
-    event = Event(Gdk.EventType.BUTTON_RELEASE, button=1)
+    event = create_event(Gdk.EventType.BUTTON_RELEASE, button=1)
     with mock.patch.object(Gtk, "get_event_widget") as get_event_widget:
         get_event_widget.return_value = clip.ui
         clip.ui._button_release_event_cb(None, event)
@@ -472,107 +460,108 @@ def select_clips(scenario, action):
                                    "Clip %s should be UNselected but is not"
                                    % clip.get_name())
     else:
-        for l in timeline.get_layers():
-            for c in l.get_clips():
-                if c.get_name() in selection:
-                    if not c.ui.get_state_flags() & Gtk.StateFlags.SELECTED:
+        for layer in timeline.get_layers():
+            for clip in layer.get_clips():
+                if clip.get_name() in selection:
+                    if not clip.ui.get_state_flags() & Gtk.StateFlags.SELECTED:
                         scenario.report_simple(GLib.quark_from_string("scenario::execution-error"),
                                                "Clip %s should be selected (as defined in selection %s)"
                                                " but is not" % (selection, clip.get_name()))
                 else:
-                    if c.ui.get_state_flags() & Gtk.StateFlags.SELECTED:
+                    if clip.ui.get_state_flags() & Gtk.StateFlags.SELECTED:
                         scenario.report_simple(GLib.quark_from_string("scenario::execution-error"),
                                                "Clip %s should NOT be selected (as defined in selection %s)"
                                                " but it is" % (selection, clip.get_name()))
 
     if mode == "ctrl":
-        event = Event(Gdk.EventType.KEY_RELEASE, keyval=Gdk.KEY_Control_L)
+        event = create_event(Gdk.EventType.KEY_RELEASE, keyval=Gdk.KEY_Control_L)
         timeline.ui.get_parent().do_key_release_event(event)
 
     return 1
 
 
-def Parametter(name, desc, mandatory=False, possible_variables=None, types=None):
-    p = GstValidate.ActionParameter()
-    p.description = desc
-    p.mandatory = mandatory
-    p.name = name
-    p.possible_variables = possible_variables
-    p.types = types
+def create_action_parameter(name, desc, mandatory=False, possible_variables=None, types=None):
+    parameter = GstValidate.ActionParameter()
+    parameter.description = desc
+    parameter.mandatory = mandatory
+    parameter.name = name
+    parameter.possible_variables = possible_variables
+    parameter.types = types
 
-    return p
+    return parameter
 
 
 def init():
     global has_validate
     try:
-        from gi.repository import GstValidate
         GstValidate.init()
         has_validate = GES.validate_register_action_types()
         GstValidate.register_action_type("stop", "pitivi",
-                                         stop, None,
+                                         stop_func, None,
                                          "Pitivi override for the stop action",
                                          GstValidate.ActionTypeFlags.NONE)
 
         GstValidate.register_action_type("seek", "pitivi",
-                                         seek, None,
+                                         seek_func, None,
                                          "Pitivi override for the seek action",
                                          GstValidate.ActionTypeFlags.NONE)
 
         GstValidate.register_action_type("pause", "pitivi",
-                                         set_state, None,
+                                         set_state_func, None,
                                          "Pitivi override for the pause action",
                                          GstValidate.ActionTypeFlags.NONE)
 
         GstValidate.register_action_type("play", "pitivi",
-                                         set_state, None,
+                                         set_state_func, None,
                                          "Pitivi override for the pause action",
                                          GstValidate.ActionTypeFlags.NONE)
 
         GstValidate.register_action_type("set-state", "pitivi",
-                                         set_state, None,
+                                         set_state_func, None,
                                          "Pitivi override for the set-state action",
                                          GstValidate.ActionTypeFlags.NONE)
 
         GstValidate.register_action_type("edit-container", "pitivi",
-                                         editContainer, None,
+                                         edit_container_func, None,
                                          "Start dragging a clip in the timeline",
                                          GstValidate.ActionTypeFlags.NONE)
 
         GstValidate.register_action_type("split-clip", "pitivi",
-                                         split_clip, None,
+                                         split_clip_func, None,
                                          "Split a clip",
                                          GstValidate.ActionTypeFlags.NONE)
 
         GstValidate.register_action_type("add-layer", "pitivi",
-                                         add_layer, None,
+                                         add_layer_func, None,
                                          "Add layer",
                                          GstValidate.ActionTypeFlags.NONE)
 
         GstValidate.register_action_type("remove-clip", "pitivi",
-                                         remove_clip, None,
+                                         remove_clip_func, None,
                                          "Remove clip",
                                          GstValidate.ActionTypeFlags.NONE)
         GstValidate.register_action_type("select-clips", "pitivi",
-                                         select_clips, [Parametter("clip-name",
-                                                                   "The name of the clip to select",
-                                                                   True, None, "str")],
+                                         select_clips_func,
+                                         [create_action_parameter("clip-name",
+                                                                  "The name of the clip to select",
+                                                                  True, None, "str")],
                                          "Select clips",
                                          GstValidate.ActionTypeFlags.NONE)
 
-        for z in ["zoom-fit", "zoom-out", "zoom-in"]:
-            GstValidate.register_action_type(z, "pitivi", zoom, None, z,
+        for action_type in ["zoom-fit", "zoom-out", "zoom-in"]:
+            GstValidate.register_action_type(action_type, "pitivi",
+                                             zoom_func, None, action_type,
                                              GstValidate.ActionTypeFlags.NO_EXECUTION_NOT_FATAL)
-        GstValidate.register_action_type('set-zoom-level', "pitivi", setZoomLevel, None, z,
+        GstValidate.register_action_type('set-zoom-level', "pitivi", set_zoom_level_func, None, action_type,
                                          GstValidate.ActionTypeFlags.NO_EXECUTION_NOT_FATAL)
 
         Gst.info("Adding pitivi::wrong-window-creation")
         GstValidate.Issue.register(GstValidate.Issue.new(
-                                   GLib.quark_from_string("pitivi::wrong-window-creation"),
-                                   "A new window for the sink has wrongly been created",
-                                   "All sink should display their images in an embedded "
-                                   "widget and thus not create a new window",
-                                   GstValidate.ReportLevel.CRITICAL))
+            GLib.quark_from_string("pitivi::wrong-window-creation"),
+            "A new window for the sink has wrongly been created",
+            "All sink should display their images in an embedded "
+            "widget and thus not create a new window",
+            GstValidate.ReportLevel.CRITICAL))
         return True
     except ImportError:
         has_validate = False
