@@ -13,9 +13,7 @@
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
-# License along with this program; if not, write to the
-# Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
-# Boston, MA 02110-1301, USA.
+# License along with this program; if not, see <http://www.gnu.org/licenses/>.
 """Tests for the utils.pipeline module."""
 # pylint: disable=protected-access,no-self-use
 from unittest import mock
@@ -41,7 +39,7 @@ class TestPipeline(common.TestCase):
         error = GLib.Error.new_literal(Gst.core_error_quark(),
                                        "fake", Gst.CoreError.TOO_LAZY)
         message.parse_error = mock.MagicMock(return_value=(error, "fake"))
-        pipeline._busMessageCb(None, message)
+        pipeline._bus_message_cb(None, message)
 
     def test_recovery(self):
         """Checks the recovery mechanism."""
@@ -94,7 +92,7 @@ class TestPipeline(common.TestCase):
         # We mock set_state because we don't actually care about the state,
         # and setting the state to PAUSED could show a video window.
         with mock.patch.object(pipe, "set_state"):
-            pipe._async_done_not_received_cb()
+            pipe._async_done_not_received_cb("reason1", 1)
         # Make sure the pipeline started a watchdog timer waiting for async-done
         # as part of setting the state from NULL to PAUSED.
         self.assertTrue(pipe._busy_async)
@@ -106,12 +104,12 @@ class TestPipeline(common.TestCase):
         message.type = Gst.MessageType.STATE_CHANGED
         message.src = pipe._pipeline
         message.parse_state_changed.return_value = (Gst.State.NULL, Gst.State.READY, Gst.State.PAUSED)
-        pipe._busMessageCb(None, message)
+        pipe._bus_message_cb(None, message)
 
         # Pretend the state changed to PAUSED.
         message.parse_state_changed.return_value = (Gst.State.READY, Gst.State.PAUSED, Gst.State.VOID_PENDING)
         self.assertEqual(pipe._next_seek, None)
-        pipe._busMessageCb(None, message)
+        pipe._bus_message_cb(None, message)
         self.assertEqual(pipe._recovery_state, SimplePipeline.RecoveryState.SEEKED_AFTER_RECOVERING)
         self.assertTrue(pipe._busy_async)
         # The pipeline should have tried to seek back to the last position.
@@ -121,7 +119,7 @@ class TestPipeline(common.TestCase):
         message.type = Gst.MessageType.ASYNC_DONE
         with mock.patch.object(pipe, "get_state") as get_state:
             get_state.return_value = (0, Gst.State.PAUSED, 0)
-            pipe._busMessageCb(None, message)
+            pipe._bus_message_cb(None, message)
         self.assertEqual(pipe._recovery_state, SimplePipeline.RecoveryState.NOT_RECOVERING)
         # Should still be busy because of seeking to _next_seek.
         self.assertTrue(pipe._busy_async)
@@ -129,7 +127,29 @@ class TestPipeline(common.TestCase):
 
         # Pretend the seek async operation finished.
         message.type = Gst.MessageType.ASYNC_DONE
-        pipe._busMessageCb(None, message)
+        pipe._bus_message_cb(None, message)
         self.assertEqual(pipe._recovery_state, SimplePipeline.RecoveryState.NOT_RECOVERING)
         self.assertFalse(pipe._busy_async)
         self.assertIsNone(pipe._next_seek)
+
+    def test_commit_timeline_after(self):
+        """Checks the recovery mechanism."""
+        pipe = Pipeline(common.create_pitivi_mock())
+        timeline = GES.Timeline()
+        pipe.set_timeline(timeline)
+
+        with mock.patch.object(pipe, "get_state") as get_state:
+            get_state.return_value = (0, Gst.State.PAUSED, 0)
+            with mock.patch.object(timeline, "commit") as commit:
+                with pipe.commit_timeline_after():
+                    pipe.commit_timeline()
+                self.assertEqual(commit.call_count, 1)
+
+            with mock.patch.object(timeline, "commit") as commit:
+                with pipe.commit_timeline_after():
+                    self.assertEqual(pipe._prevent_commits, 1)
+                    with pipe.commit_timeline_after():
+                        self.assertEqual(pipe._prevent_commits, 2)
+                        pipe.commit_timeline()
+                        self.assertEqual(commit.call_count, 0)
+                self.assertEqual(commit.call_count, 1)
