@@ -13,11 +13,8 @@
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
-# License along with this program; if not, write to the
-# Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
-# Boston, MA 02110-1301, USA.
+# License along with this program; if not, see <http://www.gnu.org/licenses/>.
 """Tests for the utils.system module."""
-# pylint: disable=missing-docstring
 from unittest import TestCase
 
 from pitivi.utils.system import System
@@ -25,12 +22,12 @@ from pitivi.utils.system import System
 
 class TestSystem(TestCase):
 
-    def testGetUniqueFilename(self):
+    def test_get_unique_filename(self):
         system = System()
-        self.assertNotEqual(system.getUniqueFilename("a/b"),
-                            system.getUniqueFilename("a%47b"))
-        self.assertNotEqual(system.getUniqueFilename("a%b"),
-                            system.getUniqueFilename("a%37b"))
-        self.assertNotEqual(system.getUniqueFilename("a%/b"),
-                            system.getUniqueFilename("a%37%3747b"))
-        self.assertEqual("a b", system.getUniqueFilename("a b"))
+        self.assertNotEqual(system.get_unique_filename("a/b"),
+                            system.get_unique_filename("a%47b"))
+        self.assertNotEqual(system.get_unique_filename("a%b"),
+                            system.get_unique_filename("a%37b"))
+        self.assertNotEqual(system.get_unique_filename("a%/b"),
+                            system.get_unique_filename("a%37%3747b"))
+        self.assertEqual("a b", system.get_unique_filename("a b"))
