@@ -156,8 +156,8 @@ class TestTimelineObserver(common.TestCase):
 
         self.assertTrue(self.layer.add_clip(clip1))
         self.assertTrue(self.layer.add_clip(clip2))
-        self.assertEqual(clip1.props.timeline, self.layer.get_timeline())
-        self.assertEqual(clip2.props.timeline, self.layer.get_timeline())
+        self.assertEqual(clip1.get_timeline(), self.layer.get_timeline())
+        self.assertEqual(clip2.get_timeline(), self.layer.get_timeline())
 
         self.timeline_container.timeline.selection.select([clip1, clip2])
         self.timeline_container.group_action.activate(None)
@@ -784,8 +784,7 @@ class TestControlSourceObserver(common.TestCase):
         asset = GES.UriClipAsset.request_sync(uri)
         clip = asset.extract()
         self.layer.add_clip(clip)
-        source = clip.get_children(False)[1]
-        self.assertTrue(isinstance(source, GES.VideoUriSource))
+        source = self.get_clip_element(clip, GES.VideoUriSource)
 
         control_source = GstController.InterpolationControlSource()
         control_source.props.mode = GstController.InterpolationMode.LINEAR
@@ -810,8 +809,7 @@ class TestControlSourceObserver(common.TestCase):
         asset = GES.UriClipAsset.request_sync(uri)
         clip = asset.extract()
         self.layer.add_clip(clip)
-        source = clip.get_children(False)[1]
-        self.assertTrue(isinstance(source, GES.VideoUriSource))
+        source = self.get_clip_element(clip, GES.VideoUriSource)
 
         control_source = GstController.InterpolationControlSource()
         control_source.props.mode = GstController.InterpolationMode.LINEAR
@@ -839,8 +837,7 @@ class TestControlSourceObserver(common.TestCase):
         asset = GES.UriClipAsset.request_sync(uri)
         clip = asset.extract()
         self.layer.add_clip(clip)
-        source = clip.get_children(False)[1]
-        self.assertTrue(isinstance(source, GES.VideoUriSource))
+        source = self.get_clip_element(clip, GES.VideoUriSource)
 
         control_source = GstController.InterpolationControlSource()
         control_source.props.mode = GstController.InterpolationMode.LINEAR
