@@ -107,27 +107,27 @@ class TestSelection(common.TestCase):
         selection = Selection()
 
         selection.set_selection([clip1, clip2, clip3, clip4], SELECT)
-        self.assertSetEqual(selection.toplevels, {clip1, clip2, clip3, clip4})
+        self.assertSetEqual(selection.toplevels(), {clip1, clip2, clip3, clip4})
 
         group1 = GES.Container.group([clip1, clip2])
         group1.props.serialize = True
-        self.assertSetEqual(selection.toplevels, {group1, clip3, clip4})
+        self.assertSetEqual(selection.toplevels(), {group1, clip3, clip4})
 
         group2 = GES.Container.group([group1, clip3])
         group2.props.serialize = True
-        self.assertSetEqual(selection.toplevels, {group2, clip4})
+        self.assertSetEqual(selection.toplevels(), {group2, clip4})
 
         group1.props.serialize = True
         group1.props.serialize = False
-        self.assertSetEqual(selection.toplevels, {group2, clip4})
+        self.assertSetEqual(selection.toplevels(), {group2, clip4})
 
         group1.props.serialize = False
         group2.props.serialize = False
-        self.assertSetEqual(selection.toplevels, {clip1, clip2, clip3, clip4})
+        self.assertSetEqual(selection.toplevels(), {clip1, clip2, clip3, clip4})
 
         group1.props.serialize = True
         group2.props.serialize = False
-        self.assertSetEqual(selection.toplevels, {group1, clip3, clip4})
+        self.assertSetEqual(selection.toplevels(), {group1, clip3, clip4})
 
 
 class TestEditingContext(common.TestCase):
@@ -135,9 +135,9 @@ class TestEditingContext(common.TestCase):
 
     def test_with_video(self):
         """Checks the value of the with_video field."""
-        audio_clip = common.get_sample_clip("mp3_sample.mp3")
-        video_clip = common.get_sample_clip("one_fps_numeroted_blue.mkv")
-        audio_video_clip = common.get_sample_clip("tears_of_steel.webm")
+        audio_clip = GES.UriClipAsset.request_sync(common.get_sample_uri("mp3_sample.mp3")).extract()
+        video_clip = GES.UriClipAsset.request_sync(common.get_sample_uri("one_fps_numeroted_blue.mkv")).extract()
+        audio_video_clip = GES.UriClipAsset.request_sync(common.get_sample_uri("tears_of_steel.webm")).extract()
 
         # Add the clips to a layer so they have TrackElements.
         project = common.create_project()
