@@ -13,15 +13,15 @@
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
-# License along with this program; if not, write to the
-# Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
-# Boston, MA 02110-1301, USA.
+# License along with this program; if not, see <http://www.gnu.org/licenses/>.
+from gi.repository import Gdk
 from gi.repository import Gst
 
 from pitivi.utils.widgets import ChoiceWidget
 from pitivi.utils.widgets import ColorWidget
 from pitivi.utils.widgets import FontWidget
 from pitivi.utils.widgets import FractionWidget
+from pitivi.utils.widgets import GstElementSettingsDialog
 from pitivi.utils.widgets import NumericWidget
 from pitivi.utils.widgets import PathWidget
 from pitivi.utils.widgets import TextWidget
@@ -31,7 +31,7 @@ from tests import common
 
 class TestWidgets(common.TestCase):
 
-    def testConstruction(self):
+    def test_construction(self):
         widgets = (
             (PathWidget, "file:///home/", ()),
             (TextWidget, "banana", ()),
@@ -41,37 +41,49 @@ class TestWidgets(common.TestCase):
                 ("banana", "banana"),
                 ("apple", "apple"),
                 ("pear", "pear")),)),
-            (ColorWidget, 0x336699FF, (int,)),
+            (ColorWidget, Gdk.RGBA(0.5, 0.5, 0.3, 0.8), ()),
             (FontWidget, "Sans 9", ()))
 
         for widget_class, default, args in widgets:
             widget = widget_class(*args, default=default)
-            self.assertEqual(default, widget.getWidgetDefault())
-            widget.setWidgetToDefault()
-            self.assertEqual(default, widget.getWidgetValue())
-            widget.setWidgetValue(default)
-            self.assertEqual(default, widget.getWidgetValue())
+            self.assertEqual(default, widget.get_widget_default())
+            widget.set_widget_to_default()
+            self.assertEqual(default, widget.get_widget_value())
+            widget.set_widget_value(default)
+            self.assertEqual(default, widget.get_widget_value())
 
-    def testValidation(self):
-        widget = TextWidget("^([a-zA-Z]+\s*)+$")
+    def test_validation(self):
+        widget = TextWidget("^([a-zA-Z]+\\s*)+$")
         bad_value = "1"
-        self.assertNotEqual(bad_value, widget.getWidgetValue())
+        self.assertNotEqual(bad_value, widget.get_widget_value())
 
-        widget = TextWidget("^\d+$", ("12", "14"))
+        widget = TextWidget("^\\d+$", ("12", "14"))
         bad_value = "non-digits"
-        self.assertNotEqual(bad_value, widget.getWidgetValue())
+        self.assertNotEqual(bad_value, widget.get_widget_value())
 
 
 class TestFractionWidget(common.TestCase):
 
     def test_widget_text(self):
         widget = FractionWidget()
-        widget.setWidgetValue(Gst.Fraction(1000000, 1))
+        widget.set_widget_value(Gst.Fraction(1000000, 1))
         self.assertEqual(widget.text.get_text(), "1000000:1")
-        widget.setWidgetValue(Gst.Fraction(7504120000000001, 4503600000000002))
+        widget.set_widget_value(Gst.Fraction(7504120000000001, 4503600000000002))
         self.assertEqual(widget.text.get_text(), "7504120000000001:4503600000000002")
 
-    def test_widget_text_magic_M(self):
+    def test_widget_text_magic__m(self):
         widget = FractionWidget()
-        widget.setWidgetValue(Gst.Fraction(1000000000, 1001))
+        widget.set_widget_value(Gst.Fraction(1000000000, 1001))
         self.assertEqual(widget.text.get_text(), "1000000M")
+
+
+class TestGstElementSettingsDialog(common.TestCase):
+
+    def test_reusing_properties(self):
+        """Checks passing values to be used on element to be configured works."""
+        values = {"datarate": 12}
+        dialog = GstElementSettingsDialog(Gst.ElementFactory.find("identity"),
+                                          values)
+        widgets = {prop.name: widget
+                   for prop, widget in dialog.elementsettings.properties.items()}
+        self.assertEqual(widgets["datarate"].get_widget_value(), 12)
